from waapi import WaapiClient, CannotConnectToWaapiException
from pprint import pprint
import argparse, os, sys, socket

# Define arguments for the script
parser = argparse.ArgumentParser(description='Render the current GameSynth patch in a Wwise container')
parser.add_argument('id', metavar='GUID', nargs='+', help='one or multiple guid of the form \{01234567-89ab-cdef-0123-4567890abcde\}')
parser.add_argument('--original', help='path to the original folder',required=True)

args = parser.parse_args()

# Settings to send message to the GSAPI
port = 28542
delimiter = "\r"

try:
    # Connecting to Waapi using default URL
    with WaapiClient() as client:

        # Connect to the GameSynth Tool API
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.connect(('localhost', port))
            print("*** Connected to the GameSynth Tool API ***")

        except ConnectionRefusedError:
            print("Could not connect to GameSynth, please start GameSynth first.")
            sys.exit()

        # Get the name of the current GameSynth patch
        s.send(("get_patchname" + delimiter).encode('utf-8'))
        gs_patch_name = s.recv(port).decode('utf-8')
        gs_patch_name = gs_patch_name.rstrip(delimiter)
        if "ERROR: no patch loaded" in gs_patch_name:
            print("GSAPI " + gs_patch_name)
            s.close()
            print("*** Disconnected from the GameSynth Tool API ***")
            sys.exit()

        # Check if the patch is infinite
        s.send("is_infinite".encode('utf-8'))
        is_infinite_result = s.recv(port).decode('utf-8')
        is_infinite_result = int(str(is_infinite_result).replace(delimiter, ""))

        # Initialize variables to hold the parsed values
        CHANNELS, BITDEPTH, DURATION, VARIATIONS, wwise_loop = None, None, None, None, None

        # Define a mapping for CHANNELS and BITDEPTH
        CHANNELS_mapping = {"Mono": 1, "Stereo": 2}
        BITDEPTH_mapping = {"16-bit": 16, "24-bit": 24}

        # Preparing GSAPI window parameters
        window_label = "{LABEL, \"Select the Rendering Settings\", HEADER, CENTER}"
        patchname_string = '{{STRING, "Name of the sound", NORMAL, "{0}"}}'.format(gs_patch_name)
        variations_int = "{NUMBER, \"Number of Variations\", INTEGER, \"\", 1, 30, 1, 0}"
        bitdepth_enum = "{ENUM, \"Bit depth\", COMBO, \"16-bit,24-bit\", 16-bit}"
        channels_enum = "{ENUM, \"Channels\", COMBO, \"Mono,Stereo\", Mono}"
        duration_float = "{NUMBER, \"Duration\", FLOAT, \"s\", 0.1, 180, 10, 1}"
        wwise_loop_bool = "{BOOL, \"Create Loop in Sound SFX\", TRUE}"

        # Preparing GSAPI message
        gsapi_window_parameters = f"window_parameters {window_label} {patchname_string} {variations_int} {bitdepth_enum} {channels_enum}"
        if is_infinite_result == 1:     
            gsapi_window_parameters += f" {duration_float} {wwise_loop_bool}" # add duration and loop parameters if patch is infinite

        # Set timeout
        s.settimeout(None)  # should be by default, but having some other tools open in the background can override it
        # Open GSAPI window to get user input
        s.send((gsapi_window_parameters + delimiter).encode('utf-8'))

        window_message_result = s.recv(port).decode('utf-8')

        window_message_result = window_message_result.rstrip(delimiter)
        # Split the received message into parts based on commas
        rendering_parameters = window_message_result.split(',')
        
        # Check if there are enough items
        if len(rendering_parameters) == 4:
            # Assign each item to the corresponding variable
            gs_patch_name, VARIATIONS, BITDEPTH, CHANNELS = rendering_parameters
            DURATION = 0
            wwise_loop = "FALSE"
        elif len(rendering_parameters) == 6:
            gs_patch_name, VARIATIONS, BITDEPTH, CHANNELS, DURATION, wwise_loop = rendering_parameters
        else:
            s.close()
            print("*** Disconnected from the GameSynth Tool API ***")
            print("Some rendering parameters are missing.")
            sys.exit()

        # Change CHANNELS, BITDEPTH, DURATION, VARIATIONS to the desired format
        CHANNELS = CHANNELS_mapping[CHANNELS]
        BITDEPTH = BITDEPTH_mapping[BITDEPTH]
        DURATION = float(DURATION)
        VARIATIONS = int(VARIATIONS)

        # Stop the script if the patch is infinite and the duration is 0
        if (is_infinite_result == 1 and DURATION <= 0):
            print("GSAPI: The patch is infinite, please set a duration higher than 0.")
            s.close()
            print("*** Disconnected from the GameSynth Tool API ***")
            sys.exit()

        # Get the path of the Wwise Originals directory
        originals_subfolder = "SFX"
        originals_path = os.path.join(args.original, originals_subfolder)

        # Obtain more information about the objects being passed
        get_args = {
            "from": {"id": args.id},
        }
        options = {
            "return": ['name', 'type', 'path', 'parent']
        }
        get_result = client.call("ak.wwise.core.object.get", get_args, options=options)

        imports = []

        for variation_index in range(VARIATIONS):
            for obj in get_result['return']:
                
                # Append the index to gs_patch_name and build the render path
                gs_patch_name_with_index = f"{gs_patch_name}_{variation_index}"
                gs_render_path = os.path.join(originals_path, gs_patch_name_with_index) + '.wav'

                # Prepare the render_patch command
                gsapi_render_patch = f"render_patch \"{gs_render_path}\" {BITDEPTH} {CHANNELS} {DURATION}"
                print(f"GSAPI Command: \"{gsapi_render_patch}\"")

                # Send the render_patch command
                render_patch = s.send((gsapi_render_patch + delimiter).encode('utf-8'))
                render_patch_result = s.recv(port).decode('utf-8')
                render_patch_result = render_patch_result.rstrip(delimiter)
                if "OK" not in render_patch_result:
                    print("GSAPI: " + render_patch_result)
                    s.close()
                    print("*** Disconnected from the GameSynth Tool API ***")
                    sys.exit()
                else:
                    print(f"GSAPI: \"{gs_patch_name_with_index}\" has been rendered.")

                # If the selected object is not a Sound SFX, use the patch name for the new Sound SFX
                if "Sound" not in obj['type']:
                    obj['name'] = gs_patch_name_with_index

                # Append the Sound container parameters in the "imports" list
                imports = [
                    { 
                        "audioFile": gs_render_path,
                        "objectPath": obj['path'] + '\\<AudioFileSource>' + obj['name'],
                        "importLanguage": "SFX"
                    }
                ]

                # Prepare arguments and options for the import
                import_args = {
                    "importOperation": "useExisting",
                    "default": {},
                    "imports": imports
                }
                import_opts = {
                    "return": ['id', 'name', 'type', 'parent', 'activeSource.id']
                }

                # Import the generated wave files into Wwise
                import_audio = client.call("ak.wwise.core.audio.import",import_args, options=import_opts)
                #pprint(import_audio)

                # Change the relevant properties of the Sound container and AudioFileSource if the user want to create a loop in Wwise
                if "TRUE" in wwise_loop:
                    # Extract the 'type' and define IDs from the 'import_audio' dictionary
                    for audio_object in import_audio['objects']:
                        imported_type = audio_object['type']
                        
                        if "AudioFileSource" not in imported_type:
                            imported_id = audio_object['id']  # Extract the 'id' from the current audio_object
                            imported_source_id = audio_object['activeSource.id']
                        else:
                            imported_id = audio_object['parent']['id']  # Extract the 'parent id' from the current audio_object
                            imported_source_id = audio_object['id']

                    properties_to_set = [
                        {
                            "object": imported_id,
                            "property": "IsLoopingEnabled",
                            "value": True
                        },
                        {
                            "object": imported_source_id,
                            "property": "OverrideWavLoop",
                            "value": True
                        },
                        {
                            "object": imported_source_id,
                            "property": "LoopBegin",
                            "value": DURATION * 0.10
                        },
                        {
                            "object": imported_source_id,
                            "property": "LoopEnd",
                            "value": DURATION * 0.90
                        },
                        {
                            "object": imported_source_id,
                            "property": "CrossfadeDuration",
                            "value": ((DURATION * 1000) * 0.05)
                        }
                    ]

                    # Loop through the list of properties and make the client calls
                    for prop in properties_to_set:
                        client.call("ak.wwise.core.object.setProperty", prop)

            # Prevent the generation of more than 1 file within a Sound SFX
            if (("Sound" in obj['type']) and (VARIATIONS > 1)):
                print("Cannot import more than 1 file in a Sound SFX. Variations generation has been aborted.")
                break

        # Disconnect from the GameSynth Tool API
        s.close()
        print("*** Disconnected from the GameSynth Tool API ***")

except CannotConnectToWaapiException:
    print("Could not connect to Waapi: Is Wwise running and Wwise Authoring API enabled?")